
/**
 * Title:        Diff Word<p>
 * Description:  A diff that is not line based but word based.<p>
 * Copyright:    Copyright (c) Baptiste Lepilleur<p>
 * Company:      Gaia Creation<p>
 * @author Baptiste Lepilleur
 * @version 1.0
 */
package gaiacrtn.diffword;
import java.util.List;
import java.io.PrintStream;
import gaiacrtn.util.ChangeStatus;
import gaiacrtn.util.ChangeBuilder;

public class ChangeBuilderMarkerHtml extends ChangeBuilderMarker
                                     implements SwitchTextMarker, ChangeStatus {
  protected int _nChangeStatus;
  protected PrintStream _shtml;

  // HTML with PRE text
  private String _strHtmlBegin = "<HTML><BODY><PRE>";
  private String _strHtmlEnd = "\n</PRE></BODY></HTML>";
  private String [] _strHtmlBeginChange = {
      "", "<STRIKE style=\"color:blue\">", "<B style=\"color:red\">"
    };
  private String [] _strHtmlEndChange = {
      "", "</STRIKE>", "</B>"
    };
  private String [] _strHtmlEndLine = {
//      "\n", "<I>&lt;CR&gt</I>", "<I>&lt;CR&gt</I>\n"
      "\n", "", "\n"
    };

/* // HTML with paragraph preformated
  private String _strHtmlBegin = "<HTML><BODY>";
  private String _strHtmlEnd = "\n</BODY></HTML>";
  private String [] _strHtmlBeginChange = {
      "", "<STRIKE style=\"color:blue\">", "<B style=\"color:red\">"
    };
  private String [] _strHtmlEndChange = {
      "", "</STRIKE>", "</B>"
    };
  private String [] _strHtmlEndLine = {
      "<P>&nbsp;&nbsp;&nbsp;&nbsp;\n", "", "<P>&nbsp;&nbsp;&nbsp;&nbsp;\n"
    };
*/

  /** Creates a change builder that build an HTML document.
   * @param shtml The HTML codes are printed in this stream.
   */
  public ChangeBuilderMarkerHtml( PrintStream shtml) {
    _shtml = shtml;
  }

  public void begin(List listMarker0, List listMarker1) {
      _shtml.println( _strHtmlBegin );
  }

  public void end() {
      _shtml.println( _strHtmlEnd );
  }

  public void beginChange( List listMarker, int nStatus ) {
    _nChangeStatus = nStatus;
    _shtml.print( _strHtmlBeginChange[ _nChangeStatus ] );
  }

  public void endChange() {
    _shtml.print( _strHtmlEndChange[ _nChangeStatus ] );
  }

  public void caseWord(TextMarkerWord marker) {
    _shtml.print( marker );
  }

  public void caseEndLine(TextMarkerEndLine marker) {
    _shtml.print( _strHtmlEndLine[ _nChangeStatus ] );
  }
}