/**
 * Title:        Diff Word<p>
 * Description:  A diff that is not line based but word based.<p>
 * Copyright:    Copyright (c) Baptiste Lepilleur<p>
 * Company:      Gaia Creation<p>
 * @author Baptiste Lepilleur
 * @version 1.0
 */

package gaiacrtn.diffword;
import java.util.*;
import java.io.*;
import java.text.*;
import gaiacrtn.util.Diff;
import gaiacrtn.util.CollectionManip;
import gaiacrtn.util.TextReader;
import gaiacrtn.util.ChangeBuilder;
import gaiacrtn.util.ChangeDirector;
import gaiacrtn.util.ChangeDirector1To2;
import gaiacrtn.util.ChangeDirector2To1;
import gaiacrtn.swing.ModelString;
import gaiacrtn.swing.text.DocumentHelper;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.HTMLDocument;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dimension;
import gaiacrtn.util.PropertiesApp;
import gaiacrtn.util.ResourceApp;
import gaiacrtn.swing.MenuDefFactory;
import gaiacrtn.swing.MenuDefFactoryXMenu;
import gaiacrtn.swing.CommandHolder;
import gaiacrtn.swing.MenuMaker;
import gaiacrtn.util.PropertiesUser;
import java.util.StringTokenizer;
import java.awt.Color;
import javax.swing.text.Document;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

public class DiffWordApp {
  final static String PRP_LOCALE_FORCE = "locale.force";
  final static String PRP_LOCALE_NAME = "locale.name";
  final static String PRP_DIFF_SAVE_HTML_FILENAME = "diff.save.html.filename";
  final static String PRP_DIFF_LOAD_FILENAME0 = "diff.load.filename0";
  final static String PRP_DIFF_LOAD_FILENAME1 = "diff.load.filename1";

  PropertiesUser _prpUser;

  ModelFilename _modelFilename0;
  ModelFilename _modelFilename1;

  void setupApp() {
    PropertiesApp.setPrp( PropertiesApp.APP_NAME, "DiffWord" );
    PropertiesApp.setPrp( PropertiesApp.APP_VERSION, "1.0" );
    PropertiesApp.setPrp( PropertiesApp.APP_HOME,
                          System.getProperty( "user.dir" ) );

    // load user properties
    _prpUser = PropertiesUser.create( null );
    _prpUser.load();
    if ( _prpUser.getBoolPrp( PRP_LOCALE_FORCE, false ) )
    {
      String strLocaleName = _prpUser.getPrp( PRP_LOCALE_NAME );
      StringTokenizer tokenizer = new StringTokenizer( strLocaleName, "_" );
      if ( tokenizer.countTokens() < 1 )
        System.err.println( "Bad locale name in user properties!" );
      else
      {
        String strLanguage = tokenizer.nextToken();
        String strCountry = "";
        String strVariant = "";
        if ( tokenizer.hasMoreTokens() )
          strCountry = tokenizer.nextToken();
        if ( tokenizer.hasMoreTokens() )
          strVariant = tokenizer.nextToken();
        Locale.setDefault( new Locale( strLanguage, strCountry, strVariant ) );
      }
    }

    // load application resources
    try {
      ResourceApp.setResApp( ResourceApp.create() );
    }
    catch ( MissingResourceException e ) {
      System.err.println( "Failed to load resource file:\n" + e );
      quitInitFail();
    }

    // load xmenu application definition
    try {
      MenuDefFactory factoryMenuDef = new MenuDefFactoryXMenu(
          ResourceApp.appFile( "resources/" + PropertiesApp.appName() +
                                  "-xmenu.xml" ) );
      ResourceApp.setAppMenuDefFactory( factoryMenuDef );
    }
    catch ( IOException e ) {
      System.err.println( "Failed to load xmenu application definition:\n" +
                          e );
      quitInitFail();
    }

    // creates application command holder
    ResourceApp.setAppCommandHolder( new CommandHolder() );
  }

  public DiffWordApp() {
    setupApp();
    setupUserProperties();

    setupCommands();

    JFrame frameMain = new JFrame( "DiffWord" );
    JPanel panelFileSel0 = new DwPanelFileSelect( _modelFilename0 );
    JPanel panelFileSel1 = new DwPanelFileSelect( _modelFilename1 );
    JPanel panelContent = new JPanel();
    JButton buttonShowChange = new JButton();
    ResourceApp.cmdHolder.setup( buttonShowChange, "diff.show" );

//    panelContent.setLayout( new BorderLayout() );
    panelContent.setLayout( new FlowLayout() );
		panelContent.setBorder( BorderFactory.createEmptyBorder(4,4,4,4) );
//    panelContent.add( panelFileSel0, BorderLayout.WEST );
//    panelContent.add( panelFileSel1, BorderLayout.EAST );
    panelContent.add( panelFileSel0 );
    panelContent.add( panelFileSel1 );
    panelContent.add( buttonShowChange );

    frameMain.setContentPane( panelContent );

    frameMain.setJMenuBar( MenuMaker.makeMenuBar( "main.menubar" ) );

	  frameMain.addWindowListener( new WindowAdapter() {
  	  	public void windowClosing( WindowEvent e ) {
          quit( 0 );
	    	}
  	  } );

	  frameMain.pack();
	  frameMain.setVisible( true );
  }
  
  void setupUserProperties() {
    String strFilename = _prpUser.getPrp( PRP_DIFF_LOAD_FILENAME0 );
    if ( strFilename == null )
      strFilename = ResourceApp.appFilename( "resources" + File.separator + 
                                             "example-old.txt" );
    _modelFilename0 = new ModelFilename( strFilename );

    strFilename = _prpUser.getPrp( PRP_DIFF_LOAD_FILENAME1 );
    if ( strFilename == null )
      strFilename = ResourceApp.appFilename( "resources" + File.separator + 
                                             "example-new.txt" );
    _modelFilename1 = new ModelFilename( strFilename );
  }

  void setupCommands() {
    CommandHolder cmdHolder = ResourceApp.cmdHolder;

    cmdHolder.addCommand( "select.old", new ActionListener() {
          public void actionPerformed( ActionEvent e ) {
            _modelFilename0.chooseFilename();
          }
        } );

    cmdHolder.addCommand( "select.new", new ActionListener() {
          public void actionPerformed( ActionEvent e ) {
            _modelFilename1.chooseFilename();
          }
        } );

    cmdHolder.addCommand( "quit", new ActionListener() {
          public void actionPerformed( ActionEvent e ) {
            quit( 0 );
          }
        } );

    cmdHolder.addCommand( "diff.save", new ActionListener() {
          public void actionPerformed( ActionEvent e ) {
            DiffHolder diffHolder =
                new DiffHolder( _modelFilename0.text(),
                                _modelFilename1.text() );
            diffHolder.computeChange();
            diffSaveHtml( diffHolder );
          }
        } );

    cmdHolder.addCommand( "diff.show", new ActionListener() {
          public void actionPerformed( ActionEvent e ) {
            DiffHolder diffHolder =
                new DiffHolder( _modelFilename0.text(),
                                _modelFilename1.text() );
            diffHolder.computeChange();
            diffShowHtml( diffHolder );
          }
        } );
  }

  static public class DiffHolder {
    private List _listMarker0;
    private List _listMarker1;
    private Diff.change _change;

    /** Constructs Words List from the specified files.
     */
    DiffHolder( String strFilename0, String strFilename1 ) {
      System.out.println( "Reading text 1" );
      TextMarkerLineProcessor builderWords =
          new TextMarkerLineProcessor( new TextMarkerFactoryConcret() );
      TextReader.read( strFilename0, builderWords );
      _listMarker0= builderWords.takeMarkers();
      System.out.println( "Reading text 2" );
      TextReader.read( strFilename1, builderWords );
      _listMarker1 = builderWords.takeMarkers();
    }

    void computeChange() {
      System.out.println( "Computing changes..." );
      Diff diff = new Diff( _listMarker0.toArray(), _listMarker1.toArray() );
      _change = diff.diff_2( false );
    }

    void buildHtmlChange( PrintStream shtml ) {
      ChangeDirector directorChange = new ChangeDirector1To2(
          new ChangeBuilderMarkerHtml( shtml ) );
      directorChange.buildChange( _listMarker0, _listMarker1, _change );
    }
  }

  public void diffSaveHtml( DiffHolder diffHolder ) {
    String strFileSaveHtml =
        _prpUser.getPrp( PRP_DIFF_SAVE_HTML_FILENAME, null );
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setDialogTitle(
        ResourceApp.getResString( "diff.save.filechooser" ) );
    if ( strFileSaveHtml != null )
      fileChooser.setSelectedFile( new File( strFileSaveHtml) );
    int nOk = fileChooser.showSaveDialog( null );
    if ( nOk != JFileChooser.APPROVE_OPTION )   // Don't save
      return;

    File fileSaveHtml = fileChooser.getSelectedFile();
    _prpUser.setPrp( PRP_DIFF_SAVE_HTML_FILENAME,
                     fileSaveHtml.getAbsolutePath() );

    System.out.println( "Saving HTML output..." );
    try {
      PrintStream shtml = new PrintStream(
                            new BufferedOutputStream(
                                  new FileOutputStream( fileSaveHtml ) ) );
      diffHolder.buildHtmlChange( shtml );
      shtml.close();
    }
    catch ( IOException e ) {
      System.err.println( "Failed to save Html output:" + e );
    }
    System.out.println( "Done!" );
  }

  public void diffShowHtml( DiffHolder diffHolder ) {
    System.out.println( "Creating HTML output..." );
    try {
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      PrintStream shtml = new PrintStream(
                            new BufferedOutputStream(
                                os ) );
      diffHolder.buildHtmlChange( shtml );

      InputStream is = new ByteArrayInputStream( os.toByteArray() );
      shtml.close();
      os.close();

      System.out.println( "Creating HTML Viewer..." );

      JFrame frameEditor = new JFrame( "Diff result" );
      JEditorPane editor = new JEditorPane();
      JScrollPane paneScroll = new JScrollPane( editor );
      editor.setEditorKit( JEditorPane.createEditorKitForContentType( "text/html" ) );
      editor.setEditable( false );
      editor.read( is, new HTMLDocument() );
      JPanel panelContent = new JPanel();
      panelContent.setLayout( new BorderLayout() );
      panelContent.add( paneScroll, BorderLayout.CENTER );
      frameEditor.setContentPane( paneScroll );
  	  frameEditor.pack();
      frameEditor.setSize(600, 400);
      Dimension screen = frameEditor.getToolkit().getScreenSize();
      frameEditor.setLocation( (screen.width - frameEditor.getSize().width) / 2,
                               (screen.height - frameEditor.getSize().height) / 2);
      frameEditor.setVisible( true );
    }
    catch ( IOException e ) {
      System.err.println( "Failed creation Html output: " + e );
    }
    System.out.println( "Done!" );
  }

  public void quit( int nStatus ) {
    _prpUser.save();
    System.exit( nStatus );
  }

  public void quitInitFail() {
    System.exit( 2 );
  }

  public static void main(String args[]) {
    DiffWordApp app = new DiffWordApp();
  }

}


class ModelFilename extends ModelString {
  ModelFilename( String strFilename ) {
    super( strFilename );
  }

  ModelFilename() {
  }

  void chooseFilename() {
    String strFilename = text();
    JFileChooser fileChooser = new JFileChooser( strFilename );
    fileChooser.setSelectedFile( new File( strFilename ) );
    int nOk = fileChooser.showOpenDialog( null );
    if ( nOk == JFileChooser.APPROVE_OPTION )
      setText( fileChooser.getSelectedFile().getAbsolutePath() );
  }
}



class DwPanelFileSelect extends JPanel {
  JTextField _textfieldFilename = new JTextField( 32 );
  JButton _buttonChoose = new JButton( "..." );
  ModelFilename _model;
  Color _colorForegroundDefault = _textfieldFilename.getForeground();

  DwPanelFileSelect( ModelFilename model ) {
    _model = model;
    _textfieldFilename.setText( _model.text() );
    add( _textfieldFilename );
    add( _buttonChoose );

    setBorder( BorderFactory.createEtchedBorder() );



    _buttonChoose.addActionListener( new ActionListener() {
          public void actionPerformed( ActionEvent e ) {
            _model.chooseFilename();
          }
        } );

    ModelString2TextFieldBinder.bind( _textfieldFilename, _model );

    _textfieldFilename.getDocument().addDocumentListener(
        new DocumentListener() {
              public void changedUpdate( DocumentEvent e ) {
              }
              public void removeUpdate( DocumentEvent e ) {
                checkFilename( e.getDocument() );
              }
              public void insertUpdate( DocumentEvent e ) {
                checkFilename( e.getDocument() );
              }

              void checkFilename( Document doc ) {  // should be put in another thread
                if ( new File( DocumentHelper.getText( doc ) ).exists() )
                  _textfieldFilename.setForeground( _colorForegroundDefault );
                else
                  _textfieldFilename.setForeground( Color.red );
              }
            } );
  }
}


class ModelString2TextFieldBinder {
  /** Binds a JTextField to a ModelString.
   * The textfield is updated when the ModelString is modified, and the
   * ModelString is updated when the JTextField lost focus.
   */
  static public void bind( final JTextField textfield,
                           final ModelString model ) {
    textfield.addFocusListener( new FocusAdapter() {
          public void focusLost( FocusEvent e ) {
            model.setText( textfield.getText() );
          }
        } );
    model.addChangeListener( new ChangeListener() {
          public void stateChanged( ChangeEvent e ) {
            textfield.setText( model.text() );
          }
        } );
  }
}
