/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package gaiacrtn.swing;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.Action;
import gaiacrtn.util.ResourceApp;
import gaiacrtn.swing.MenuHelper;

public class MenuMaker {

  public MenuMaker() {
  }

  static public JMenuBar makeMenuBar( String strMenuName ) {
    return makeMenuBar( ResourceApp.factoryMenuDef,
                        ResourceApp.cmdHolder,
                        strMenuName );
  }

  static public JMenu makeMenu( String strMenuName ) {
    return makeMenu( ResourceApp.factoryMenuDef,
                     ResourceApp.cmdHolder,
                     strMenuName );
  }

  static public JPopupMenu makePopupMenu( String strMenuName ) {
    return makePopupMenu( ResourceApp.factoryMenuDef,
                          ResourceApp.cmdHolder,
                          strMenuName );
  }


  static public JMenuBar makeMenuBar( MenuDefFactory factoryMenuDef,
                                      CommandHolder cmdHolder,
                                      String strName ) {
    MenuMakerBuilderMenuBar builder = new MenuMakerBuilderMenuBar();
    return (JMenuBar)buildMenu( factoryMenuDef, cmdHolder, builder, strName );
  }

  static public JMenu makeMenu( MenuDefFactory factoryMenuDef,
                                CommandHolder cmdHolder,
                                String strName ) {
    MenuMakerBuilderMenu builder = new MenuMakerBuilderMenu( strName );
    return (JMenu)buildMenu( factoryMenuDef, cmdHolder, builder, strName );
  }

  static public JPopupMenu makePopupMenu( MenuDefFactory factoryMenuDef,
                                          CommandHolder cmdHolder,
                                          String strName ) {
    MenuMakerBuilderPopup builder = new MenuMakerBuilderPopup( strName );
    return (JPopupMenu)buildMenu( factoryMenuDef, cmdHolder, builder, strName );
  }

  /** Builds a menu.
   * @param factoryMenuDef Factory used to retreives the menu & sub-menu
   *                       definition.
   * @param cmdHolder Command holder used to create menu item & action.
   * @param builder Builder used to add menu entry to the menu.
   * @param strMenuName Name of the menu, used to retreives the menu definition
   *                    with the menu definition factory.
   * @return Menu object. The object is the one returned by builder.menu();
   */
  static protected Object buildMenu( final MenuDefFactory factoryMenuDef,
                                     final CommandHolder cmdHolder,
                                     final MenuMakerBuilder builder,
                                     final String strMenuName ) {
    MenuDefManager manager = factoryMenuDef.findMenuManager( strMenuName );
    MenuDefIterator it = manager.iterator();
//    System.err.println( "MenuMaker: " + strMenuName );
    while ( it.hasNext() )
    {
      MenuDefEntry entry = it.next();

      entry.apply( new SwitchMenuDef() {
          public void caseMenuItem( MenuDefEntryMenuItem menu ) {
//System.err.println( "MenuMaker: " + strMenuName + ", add action - " + menu.command() );
            builder.addAction( cmdHolder.actionMenu( menu.command() ) );
          }

          public void caseSubMenu( MenuDefEntrySubMenu menu ) {
//System.err.println( "MenuMaker: " + strMenuName + ", add submenu - " + menu.name() );
            builder.addSubMenu(
                (JMenu)makeMenu( factoryMenuDef, cmdHolder, menu.name() ) );
          }

          public void caseSeparator( MenuDefEntrySeparator menu ) {
//System.err.println( "MenuMaker: " + strMenuName + ", add separator");
            builder.addSeparator();
          }
        } );

    }
    return builder.menu();
  }


  /** Builder to add menu element to a menu.
   */
  static protected interface MenuMakerBuilder {
    /** Add a separator to the menu.
     */
    public void addSeparator();

    /** Add the specified action to the menu.
     * @param action Action to add to the menu
     */
    public void addAction( Action action );

    /** Add the specified sub-menu to the menu.
     * @param menuSub Sub-menu to add.
     */
    public void addSubMenu( JMenu menuSub );

    /** Gets Resulting menu object.
     * @return Menu object that has been build.
     */
    public Object menu();
  }


  /** Builder for JMenu.
   */
  static protected class MenuMakerBuilderMenu implements MenuMakerBuilder {

    private JMenu _menu;

    public MenuMakerBuilderMenu( String strMenuName ) {
      _menu = MenuHelper.createMenu( strMenuName );
    }

    public Object menu() {
      return _menu;
    }

    public void addSeparator() {
      _menu.addSeparator();
    }

    public void addAction(Action action) {
      _menu.add( action );
    }

    public void addSubMenu(JMenu menuSub) {
      _menu.add( menuSub );
    }
  }


  /** Builder for JMenuBar.
   */
  static protected class MenuMakerBuilderMenuBar implements MenuMakerBuilder {

    private JMenuBar _menu;

    public MenuMakerBuilderMenuBar() {
      _menu = new JMenuBar();
    }

    public Object menu() {
      return _menu;
    }

    public void addSeparator() {
    }

    public void addAction(Action action) {
    }

    public void addSubMenu(JMenu menuSub) {
      _menu.add( menuSub );
    }
  }


  /** Buidler for JPopupMenu
   */
  static protected class MenuMakerBuilderPopup implements MenuMakerBuilder {

    JPopupMenu _menu;

    public MenuMakerBuilderPopup( String strMenuName ) {
      _menu = MenuHelper.createPopupMenu( strMenuName );
    }

    public Object menu() {
      return _menu;
    }

    public void addSeparator() {
      _menu.addSeparator();
    }

    public void addAction(Action action) {
      _menu.add( action );
    }

    public void addSubMenu(JMenu menuSub) {
      _menu.add( menuSub );
    }
  }
}